# Generated from Glycan.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .GlycanParser import GlycanParser
else:
    from GlycanParser import GlycanParser

# This class defines a complete listener for a parse tree produced by GlycanParser.
class GlycanListener(ParseTreeListener):

    # Enter a parse tree produced by GlycanParser#start.
    def enterStart(self, ctx:GlycanParser.StartContext):
        pass

    # Exit a parse tree produced by GlycanParser#start.
    def exitStart(self, ctx:GlycanParser.StartContext):
        pass


    # Enter a parse tree produced by GlycanParser#begin.
    def enterBegin(self, ctx:GlycanParser.BeginContext):
        pass

    # Exit a parse tree produced by GlycanParser#begin.
    def exitBegin(self, ctx:GlycanParser.BeginContext):
        pass


    # Enter a parse tree produced by GlycanParser#branch.
    def enterBranch(self, ctx:GlycanParser.BranchContext):
        pass

    # Exit a parse tree produced by GlycanParser#branch.
    def exitBranch(self, ctx:GlycanParser.BranchContext):
        pass


    # Enter a parse tree produced by GlycanParser#deriv.
    def enterDeriv(self, ctx:GlycanParser.DerivContext):
        pass

    # Exit a parse tree produced by GlycanParser#deriv.
    def exitDeriv(self, ctx:GlycanParser.DerivContext):
        pass


    # Enter a parse tree produced by GlycanParser#saci.
    def enterSaci(self, ctx:GlycanParser.SaciContext):
        pass

    # Exit a parse tree produced by GlycanParser#saci.
    def exitSaci(self, ctx:GlycanParser.SaciContext):
        pass


    # Enter a parse tree produced by GlycanParser#con.
    def enterCon(self, ctx:GlycanParser.ConContext):
        pass

    # Exit a parse tree produced by GlycanParser#con.
    def exitCon(self, ctx:GlycanParser.ConContext):
        pass


    # Enter a parse tree produced by GlycanParser#add.
    def enterAdd(self, ctx:GlycanParser.AddContext):
        pass

    # Exit a parse tree produced by GlycanParser#add.
    def exitAdd(self, ctx:GlycanParser.AddContext):
        pass


    # Enter a parse tree produced by GlycanParser#fgi.
    def enterFgi(self, ctx:GlycanParser.FgiContext):
        pass

    # Exit a parse tree produced by GlycanParser#fgi.
    def exitFgi(self, ctx:GlycanParser.FgiContext):
        pass


    # Enter a parse tree produced by GlycanParser#carb.
    def enterCarb(self, ctx:GlycanParser.CarbContext):
        pass

    # Exit a parse tree produced by GlycanParser#carb.
    def exitCarb(self, ctx:GlycanParser.CarbContext):
        pass


    # Enter a parse tree produced by GlycanParser#modi.
    def enterModi(self, ctx:GlycanParser.ModiContext):
        pass

    # Exit a parse tree produced by GlycanParser#modi.
    def exitModi(self, ctx:GlycanParser.ModiContext):
        pass


    # Enter a parse tree produced by GlycanParser#qnum.
    def enterQnum(self, ctx:GlycanParser.QnumContext):
        pass

    # Exit a parse tree produced by GlycanParser#qnum.
    def exitQnum(self, ctx:GlycanParser.QnumContext):
        pass


    # Enter a parse tree produced by GlycanParser#typi.
    def enterTypi(self, ctx:GlycanParser.TypiContext):
        pass

    # Exit a parse tree produced by GlycanParser#typi.
    def exitTypi(self, ctx:GlycanParser.TypiContext):
        pass


    # Enter a parse tree produced by GlycanParser#bridge.
    def enterBridge(self, ctx:GlycanParser.BridgeContext):
        pass

    # Exit a parse tree produced by GlycanParser#bridge.
    def exitBridge(self, ctx:GlycanParser.BridgeContext):
        pass


    # Enter a parse tree produced by GlycanParser#ct.
    def enterCt(self, ctx:GlycanParser.CtContext):
        pass

    # Exit a parse tree produced by GlycanParser#ct.
    def exitCt(self, ctx:GlycanParser.CtContext):
        pass



del GlycanParser